import bpy

class AnchorTranslate(bpy.types.Macro):
    bl_idname = "object.add_empty_and_translate"
    bl_label = "Anchor Translate"
    bl_options = {'REGISTER', 'UNDO'}

class Storage:
    ob = None
    anchor = None
    snap_element = bpy.context.tool_settings.snap_element
    snap_target = bpy.context.tool_settings.snap_target

class StoreSelectedObjects(bpy.types.Operator):
    bl_idname = "object.store_selected"
    bl_label = "Store Selected Objects"
    
    def execute(self, context):
        Storage.ob=bpy.context.selected_objects
        ob=bpy.context.selected_objects
        for o in ob:
            o.select = False
        return {'FINISHED'}
    
class AddAnchor(bpy.types.Operator):
    bl_idname = "object.add_anchor"
    bl_label = "Add Anchor"
    
    def execute(self, context):
        bpy.ops.mesh.primitive_cube_add(enter_editmode = True)
        bpy.ops.mesh.select_all
        bpy.ops.mesh.delete(type ='VERT')
        bpy.ops.object.mode_set(mode='OBJECT')
        Storage.anchor = bpy.context.object
        bpy.context.tool_settings.snap_element = 'VERTEX'
        bpy.context.tool_settings.snap_target = 'ACTIVE'
        return {'FINISHED'}
    
class ReselectStoredObjects(bpy.types.Operator):
    bl_idname = "object.reselect_stored"
    bl_label = "Reselect Stored Objects"
    
    def execute(self, context):
        ob=Storage.ob
        for o in ob:
            o.select = True
        return {'FINISHED'}
    
class DeselectReselectAnchor(bpy.types.Operator):
    bl_idname = "object.deselect_reselect_anchor"
    bl_label = "Deselect Reselect Anchor"
    
    def execute(self, context):
        anchor=Storage.anchor
        anchor.select = False
        anchor.select = True
        return {'FINISHED'}
    
class DeselectStoredObjects(bpy.types.Operator):
    bl_idname = "object.deselect_stored"
    bl_label = "Deselect Stored Objects"
    
    def execute(self, context):
        ob=Storage.ob
        for o in ob:
            o.select = False
        return {'FINISHED'}
    
class DeleteAnchor(bpy.types.Operator):
    bl_idname = "object.delete_anchor"
    bl_label = "Delete Anchor"

    def execute(self, context):
        bpy.ops.object.delete('EXEC_DEFAULT')
        bpy.context.tool_settings.snap_element = Storage.snap_element
        bpy.context.tool_settings.snap_target = Storage.snap_target
        return {'FINISHED'}

bpy.utils.register_class(AnchorTranslate)
bpy.utils.register_class(StoreSelectedObjects)
bpy.utils.register_class(AddAnchor)
bpy.utils.register_class(ReselectStoredObjects)
bpy.utils.register_class(DeselectStoredObjects)
bpy.utils.register_class(DeselectReselectAnchor)
bpy.utils.register_class(DeleteAnchor)

AnchorTranslate.define("OBJECT_OT_store_selected")
AnchorTranslate.define("OBJECT_OT_add_anchor")
AnchorTranslate.define("TRANSFORM_OT_translate")
AnchorTranslate.define("OBJECT_OT_reselect_stored")
AnchorTranslate.define("OBJECT_OT_deselect_reselect_anchor")
AnchorTranslate.define("TRANSFORM_OT_translate")
AnchorTranslate.define("OBJECT_OT_deselect_stored")
AnchorTranslate.define("OBJECT_OT_delete_anchor")
AnchorTranslate.define("OBJECT_OT_reselect_stored")